@@ -12,16 +12,15 @@ from . import box
 from .events_queue import LinearEventsQueue
 from .hints import SegmentEndpoints
 from .processing import process_open_linear_queue
-from .segment import (_relate_segment as relate_segments,
-                      relate_point as relate_point_to_segment)
+from .segment import (relate_point as relate_point_to_segment,
+                      relate_segment as relate_segments)
 from .utils import to_sorted_pair
 
 
 def relate_point(multisegment: Multisegment, point: Point,
                  context: Context) -> Relation:
     return (Relation.DISJOINT
-            if all(relate_point_to_segment(segment, point,
-                                           context)
+            if all(relate_point_to_segment(segment, point, context)
                    is Relation.DISJOINT
                    for segment in multisegment.segments)
             else Relation.COMPONENT)
@@ -29,7 +28,7 @@ def relate_point(multisegment: Multisegment, point: Point,
 
 def relate_segment(multisegment: Multisegment, segment: Segment,
                    context: Context) -> Relation:
-    all_components = has_no_touch = has_no_cross = has_no_overlap = True
+    is_segment_superset = has_no_touch = has_no_cross = has_no_overlap = True
     # orientations of multisegment's segments
     # which touch given segment in the middle
     middle_touching_orientations = {}  # type: Dict[Point, Orientation]
@@ -38,18 +37,10 @@ def relate_segment(multisegment: Multisegment, segment: Segment,
     if start > end:
         start, end = end, start
     for index, sub_segment in enumerate(multisegment.segments):
-        sub_segment_start, sub_segment_end = sub_segment_endpoints = (
-            sub_segment.start, sub_segment.end)
-        relation = relate_segments(sub_segment_start, sub_segment_end, start,
-                                   end,
-                                   context)
-        if relation is Relation.COMPONENT:
+        sub_segment_endpoints = sub_segment.start, sub_segment.end
+        relation = relate_segments(sub_segment, segment, context)
+        if relation is Relation.COMPONENT or relation is Relation.EQUAL:
             return Relation.COMPONENT
-        elif relation is Relation.EQUAL:
-            return (Relation.EQUAL
-                    if (all_components
-                        and index == len(multisegment.segments) - 1)
-                    else Relation.COMPONENT)
         elif relation is Relation.COMPOSITE:
             if has_no_overlap:
                 has_no_overlap = False
@@ -62,22 +53,22 @@ def relate_segment(multisegment: Multisegment, segment: Segment,
             else:
                 components.append(to_sorted_pair(sub_segment_endpoints))
         elif relation is Relation.OVERLAP:
-            if all_components:
-                all_components = False
+            if is_segment_superset:
+                is_segment_superset = False
             if has_no_overlap:
                 has_no_overlap = False
             start, end = segment_endpoints = _subtract_segments_overlap(
                     segment_endpoints, sub_segment_endpoints)
         else:
-            if all_components:
-                all_components = False
+            if is_segment_superset:
+                is_segment_superset = False
             if has_no_overlap:
                 if relation is Relation.TOUCH:
                     if has_no_touch:
                         has_no_touch = False
                     if has_no_cross:
                         intersection = context.segments_intersection(
-                                sub_segment_start, sub_segment_end, start, end)
+                                sub_segment, segment)
                         if intersection != start and intersection != end:
                             sub_start, sub_end = sub_segment_endpoints
                             non_touched_endpoint = (sub_start
@@ -114,7 +105,7 @@ def relate_segment(multisegment: Multisegment, segment: Segment,
             if max_component_end < component_end:
                 max_component_end = component_end
         return ((Relation.EQUAL
-                 if all_components
+                 if is_segment_superset
                  else Relation.COMPONENT)
                 if (min_component_start == start
                     and max_component_end == end
@@ -122,11 +113,13 @@ def relate_segment(multisegment: Multisegment, segment: Segment,
                             or component_end == max_component_end
                             for _, component_end in components))
                 else (Relation.COMPOSITE
-                      if all_components
+                      if is_segment_superset
                       else Relation.OVERLAP))
     else:
-        return (Relation.COMPOSITE
-                if all_components
+        return ((Relation.EQUAL
+                 if start == end
+                 else Relation.COMPOSITE)
+                if is_segment_superset
                 else Relation.OVERLAP)
 
 
@@ -141,8 +134,6 @@ def _subtract_segments_overlap(minuend: SegmentEndpoints,
 
 def relate_multisegment(goal: Multisegment, test: Multisegment,
                         context: Context) -> Relation:
-    if not (goal.segments and test.segments):
-        return Relation.DISJOINT
     goal_bounding_box = context.segments_box(goal.segments)
     test_bounding_box = context.segments_box(test.segments)
     if box.disjoint_with(goal_bounding_box, test_bounding_box):
